/**
 * Copyright (c) 2000-present Liferay, Inc. All rights reserved.
 *
 * The contents of this file are subject to the terms of the Liferay Enterprise
 * Subscription License ("License"). You may not use this file except in
 * compliance with the License. You can obtain a copy of the License by
 * contacting Liferay, Inc. See the License for the specific language governing
 * permissions and limitations under the License, including but not limited to
 * distribution rights of the Software.
 *
 *
 *
 */

package com.liferay.frontend.taglib.chart.model;

import java.util.AbstractMap;
import java.util.HashMap;
import java.util.Map;
import java.util.Set;

/**
 * @author Iván Zaera Avellón
 */
public abstract class ChartObject extends AbstractMap<String, Object> {

	@Override
	public Set<Entry<String, Object>> entrySet() {
		return _properties.entrySet();
	}

	protected <T> T get(String name, Class<T> clazz) {
		return get(name, clazz, true);
	}

	protected <T> T get(String name, Class<T> clazz, boolean createIfNotFound) {
		T value = (T)_properties.get(name);

		if ((value == null) && createIfNotFound) {
			try {
				value = clazz.newInstance();
			}
			catch (Exception e) {
				throw new RuntimeException(e);
			}

			_properties.put(name, value);
		}

		return value;
	}

	protected void set(String name, Object value) {
		if (value == null) {
			throw new UnsupportedOperationException(
				"Property " + name + " has a null value");
		}

		_properties.put(name, value);
	}

	private final Map<String, Object> _properties = new HashMap<>();

}