/**
 * Copyright (c) 2000-present Liferay, Inc. All rights reserved.
 *
 * The contents of this file are subject to the terms of the Liferay Enterprise
 * Subscription License ("License"). You may not use this file except in
 * compliance with the License. You can obtain a copy of the License by
 * contacting Liferay, Inc. See the License for the specific language governing
 * permissions and limitations under the License, including but not limited to
 * distribution rights of the Software.
 *
 *
 *
 */

package com.liferay.frontend.taglib.chart.servlet.taglib.soy.base;

import com.liferay.frontend.js.loader.modules.extender.npm.NPMResolver;
import com.liferay.frontend.taglib.chart.internal.js.loader.modules.extender.npm.NPMResolverProvider;
import com.liferay.frontend.taglib.chart.model.ChartConfig;
import com.liferay.frontend.taglib.soy.servlet.taglib.TemplateRendererTag;
import com.liferay.petra.string.StringBundler;
import com.liferay.petra.string.StringPool;
import com.liferay.portal.kernel.log.Log;
import com.liferay.portal.kernel.log.LogFactoryUtil;
import com.liferay.portal.kernel.servlet.taglib.util.OutputData;
import com.liferay.portal.kernel.theme.ThemeDisplay;
import com.liferay.portal.kernel.util.GetterUtil;
import com.liferay.portal.kernel.util.PortalUtil;
import com.liferay.portal.kernel.util.StringUtil;
import com.liferay.portal.kernel.util.WebKeys;

import java.io.IOException;

import java.net.URL;

import java.util.Map;

import javax.servlet.ServletRequest;
import javax.servlet.jsp.JspWriter;

import org.osgi.framework.Bundle;
import org.osgi.framework.FrameworkUtil;

/**
 * @author Chema Balsas
 */
public abstract class BaseChartTag extends TemplateRendererTag {

	public BaseChartTag(String moduleBaseName, String templateNamespace) {
		_moduleBaseName = moduleBaseName;
		_templateNamespace = templateNamespace;
	}

	@Override
	public int doStartTag() {
		if (_templateNamespace != null) {
			setTemplateNamespace(_templateNamespace);
		}
		else {
			setTemplateNamespace("ClayChart.render");
		}

		_outputStylesheetLink();
		_outputTilesSVG();

		return super.doStartTag();
	}

	@Override
	public String getModule() {
		NPMResolver npmResolver = NPMResolverProvider.getNPMResolver();

		if (npmResolver == null) {
			return StringPool.BLANK;
		}

		return npmResolver.resolveModuleName(
			"clay-charts/lib/" + _moduleBaseName);
	}

	public void setConfig(ChartConfig chartConfig) {
		for (Map.Entry<String, Object> entry : chartConfig.entrySet()) {
			putValue(entry.getKey(), entry.getValue());
		}
	}

	public void setId(String id) {
		putValue("id", id);
	}

	private OutputData _getOutputData() {
		ServletRequest servletRequest = getRequest();

		OutputData outputData = (OutputData)servletRequest.getAttribute(
			WebKeys.OUTPUT_DATA);

		if (outputData == null) {
			outputData = new OutputData();

			servletRequest.setAttribute(WebKeys.OUTPUT_DATA, outputData);
		}

		return outputData;
	}

	private boolean _isInline() {
		ServletRequest servletRequest = getRequest();

		ThemeDisplay themeDisplay = (ThemeDisplay)servletRequest.getAttribute(
			WebKeys.THEME_DISPLAY);

		boolean xPjax = GetterUtil.getBoolean(request.getHeader("X-PJAX"));

		if (themeDisplay.isIsolated() || themeDisplay.isLifecycleResource() ||
			themeDisplay.isStateExclusive() || xPjax) {

			return true;
		}
		else {
			return false;
		}
	}

	private void _outputStylesheetLink() {
		NPMResolver npmResolver = NPMResolverProvider.getNPMResolver();

		if (npmResolver == null) {
			return;
		}

		String cssPath = npmResolver.resolveModuleName(
			"clay-charts/lib/css/main.css");

		StringBundler sb = new StringBundler(5);

		sb.append("<link href=\"");
		sb.append(PortalUtil.getPathModule());
		sb.append("/frontend-taglib-chart/node_modules/");
		sb.append(cssPath);
		sb.append("\" rel=\"stylesheet\">");

		if (_isInline()) {
			try {
				JspWriter jspWriter = pageContext.getOut();

				jspWriter.write(sb.toString());
			}
			catch (IOException ioe) {
				_log.error("Unable to output style sheet link", ioe);
			}
		}
		else {
			OutputData outputData = _getOutputData();

			outputData.setDataSB(_OUTPUT_CSS_KEY, WebKeys.PAGE_TOP, sb);
		}
	}

	private void _outputTilesSVG() {
		OutputData outputData = _getOutputData();

		NPMResolver npmResolver = NPMResolverProvider.getNPMResolver();

		if (npmResolver == null) {
			return;
		}

		String svgPath = npmResolver.resolveModuleName(
			"clay-charts/lib/svg/tiles.svg");

		Bundle bundle = FrameworkUtil.getBundle(BaseChartTag.class);

		URL url = bundle.getEntry("META-INF/resources/node_modules/" + svgPath);

		try {
			String svg = StringUtil.read(url.openStream());

			StringBundler sb = new StringBundler(svg);

			if (_isInline()) {
				try {
					JspWriter jspWriter = pageContext.getOut();

					jspWriter.write(sb.toString());
				}
				catch (IOException ioe) {
					_log.error("Unable to output svg", ioe);
				}
			}
			else {
				outputData.setDataSB(
					_OUTPUT_SVG_KEY, WebKeys.PAGE_BODY_BOTTOM, sb);
			}
		}
		catch (IOException ioe) {
		}
	}

	private static final String _OUTPUT_CSS_KEY =
		BaseChartTag.class.getName() + "_CSS";

	private static final String _OUTPUT_SVG_KEY =
		BaseChartTag.class.getName() + "_SVG";

	private static final Log _log = LogFactoryUtil.getLog(BaseChartTag.class);

	private final String _moduleBaseName;
	private final String _templateNamespace;

}